define(['exports'], function (exports) { 'use strict';

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */

    function __spreadArrays() {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    }

    function uniqueArray(currentData, newData, comparator) {
        return __spreadArrays(currentData, newData.filter(function (i) {
            return !currentData.reduce(function (acc, cur) {
                return acc || comparator(cur, i);
            }, false);
        }));
    }
    function uniqObjArrByKey(currentData, key) {
        return currentData.reduce(function (acc, cur) {
            return !!acc.find(function (i) { return i[key] === cur[key]; }) ? acc : __spreadArrays(acc, [cur]);
        }, []);
    }
    var timeoutAsync = function (timeout) { return (new Promise(function (res) { return setTimeout(res, timeout); })); };
    var timeoutAsyncRej = function (timeout) { return (new Promise(function (res, rej) { return setTimeout(function () { return rej(new Error("timeoutAsyncRej timeout:" + timeout)); }, timeout); })); };
    function uuidv4() {
        return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
            var r = Math.random() * 16 | 0, v = c == 'x' ? r : (r & 0x3 | 0x8);
            return v.toString(16);
        });
    }
    function uid() {
        return (Date.now().toString(36) + Math.random().toString(36).substr(2, 5)).toUpperCase();
        //return '_' + Math.random().toString(36).substr(2, 9);
    }
    var SimpleEventHandler = /** @class */ (function () {
        function SimpleEventHandler() {
            this.handler = {};
        }
        SimpleEventHandler.prototype.addEventListener = function (evtName, fn) {
            var _this = this;
            var id = uuidv4();
            this.handler[evtName] = __spreadArrays(this.handler[evtName] || [], [{ id: id, fn: fn }]);
            return { unwatch: function () {
                    var index = _this.handler[evtName].findIndex(function (i) { return i.id === id; });
                    _this.handler[evtName].splice(index, 1);
                }
            };
        };
        SimpleEventHandler.prototype.notify = function (name, value) {
            var evtHandlers = this.handler[name] || [];
            evtHandlers.forEach(function (handler) { return handler.fn(value); });
        };
        return SimpleEventHandler;
    }());
    function extractJSON(str) {
        var firstOpen = str.indexOf('{');
        var sliced = str.slice(firstOpen);
        // console.log(sliced)
        var i = 0;
        var j = 0;
        for (var _i = 0, sliced_1 = sliced; _i < sliced_1.length; _i++) {
            var s = sliced_1[_i];
            j++;
            if (s === '{') {
                i++;
            }
            else if (s === '}') {
                i--;
            }
            if (i === 0) {
                var os = sliced.slice(0, j);
                return JSON.parse(os);
            }
        }
    }
    function extractJSONV2(str) {
        var firstOpen = -1, firstClose, candidate;
        firstOpen = str.indexOf('{', firstOpen + 1);
        do {
            firstClose = str.lastIndexOf('}');
            // console.log('firstOpen: ' + firstOpen, 'firstClose: ' + firstClose);
            if (firstClose <= firstOpen) {
                return null;
            }
            do {
                candidate = str.substring(firstOpen, firstClose + 1);
                // console.log('candidate: ' + candidate);
                try {
                    var res = JSON.parse(candidate);
                    // console.log('...found');
                    return [res, firstOpen, firstClose + 1];
                }
                catch (e) {
                    // console.log('...failed');
                }
                firstClose = str.substr(0, firstClose).lastIndexOf('}');
            } while (firstClose > firstOpen);
            firstOpen = str.indexOf('{', firstOpen + 1);
        } while (firstOpen != -1);
    }

    function promisesReduce(promises) {
        var final = [];
        return promises.reduce(function (pre, cur) {
            return pre.then(function () {
                return cur().then(function (result) { return final.push(result); });
            });
        }, Promise.resolve())
            .then(function () { return final; });
    }

    function downFile(content, filename) {
        // 创建隐藏的可下载链接
        var eleLink = document.createElement('a');
        eleLink.download = filename;
        eleLink.style.display = 'none';
        // 字符内容转变成blob地址
        var blob = new Blob([content]);
        eleLink.href = URL.createObjectURL(blob);
        // 触发点击
        document.body.appendChild(eleLink);
        eleLink.click();
        // 然后移除
        document.body.removeChild(eleLink);
    }
    function waitUntilElementExist(selectors, xpaths, timeout, checkInterval) {
        if (xpaths === void 0) { xpaths = []; }
        if (timeout === void 0) { timeout = 10000; }
        if (checkInterval === void 0) { checkInterval = 1000; }
        return new Promise(function (res, rej) {
            var timer;
            var timeoutTimer;
            function resolve(el) {
                clearInterval(timer);
                timer = null;
                res(el);
            }
            timeoutTimer = setTimeout(function () {
                clearInterval(timer);
                timer = null;
                rej(new Error('wait until element exist timeout!'));
            }, timeout);
            timer = setInterval(function () {
                var searchEls = selectors.map(function (selector) { return document.querySelector(selector); });
                var xpathSearchEls = xpaths.map(function (xpath) { return document.evaluate(xpath, document).iterateNext(); });
                var findEl = __spreadArrays(searchEls, xpathSearchEls).find(function (i) { return i !== null; });
                if (findEl) {
                    clearTimeout(timeoutTimer);
                    timeoutTimer = null;
                    resolve(findEl);
                }
            }, checkInterval);
        });
    }
    function scrollAndExecute(_a) {
        var scrollContainer = _a.scrollContainer, _b = _a.scrollHeight, scrollHeight = _b === void 0 ? 1000 : _b, _c = _a.scrollInterval, scrollInterval = _c === void 0 ? 2000 : _c, fn = _a.fn, _d = _a.scrollTimeout, scrollTimeout = _d === void 0 ? 10000 : _d, _e = _a.scrollStopFn, scrollStopFn = _e === void 0 ? null : _e;
        var interval;
        var callFnEveryIntervalAsync = function () {
            return new Promise(function (resolve, reject) {
                interval = setInterval(function () {
                    fn();
                    scrollContainer.scroll(0, scrollContainer.scrollTop + scrollHeight);
                    if (scrollStopFn && scrollStopFn()) {
                        resolve();
                    }
                }, scrollInterval);
            });
        };
        return Promise.race([callFnEveryIntervalAsync(), timeoutAsync(scrollTimeout)])
            .then(function () {
            clearInterval(interval);
        });
    }
    /*
    usage :
    scrollAndExecute( {scrollContainer, fn: getAndConcatData, scrollHeight: 200, scrollTimeout:30000, scrollInterval:500})
        .then(() => {
            console.log(lData)
        })
        */
    function loadScript(url, callback) {
        // adding the script tag to the head as suggested before
        var head = document.getElementsByTagName('head')[0];
        var script = document.createElement('script');
        script.type = 'text/javascript';
        script.src = url;
        // then bind the event to the callback function 
        // there are several events for cross browser compatibility
        // @ts-ignore
        script.onreadystatechange = callback;
        script.onload = callback;
        // fire the loading
        head.appendChild(script);
    }
    function openNewTab(url) {
        var win = window.open(url, '_blank');
        if (win) {
            window.focus();
        }
        else {
            setTimeout(function () {
                if (win)
                    win.focus();
            }, 100);
        }
    }
    function getUidFromLocalStorage() {
        var id = window.localStorage.getItem('jsuuid');
        if (id) {
            return id;
        }
        else {
            var newUid = uid();
            window.localStorage.setItem('jsuuid', newUid);
            return newUid;
        }
    }
    function getCanvas(_a) {
        var _b = _a.w, w = _b === void 0 ? 121 : _b, _c = _a.h, h = _c === void 0 ? 14 : _c, _d = _a.ratio, ratio = _d === void 0 ? 1 : _d, canvasEl = _a.canvasEl, text = _a.text, _e = _a.x, x = _e === void 0 ? 0 : _e, _f = _a.y, y = _f === void 0 ? 14 : _f, _g = _a.fillStyle, fillStyle = _g === void 0 ? null : _g;
        // const ratio = 2
        // console.log(this.canvasRef.current)
        canvasEl.width = w * ratio; // 实际渲染像素
        canvasEl.height = h * ratio; // 实际渲染像素
        canvasEl.style.width = w + "px"; // 控制显示大小
        canvasEl.style.height = h + "px"; // 控制显示大小
        canvasEl.getContext('2d').setTransform(ratio, 0, 0, ratio, 0, 0);
        var context = canvasEl.getContext("2d");
        context.font = "13px -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, 'Helvetica Neue', Arial, 'Noto Sans', sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol', 'Noto Color Emoji'";
        context.fillStyle = fillStyle;
        context.fillText(text, x || 0, y || 14);
        // return canvas;
        return canvasEl.toDataURL('image/png', 1);
    }

    //
    // check https://stackoverflow.com/questions/3955803/chrome-extension-get-page-variables-in-content-script
    //
    var ExtractPageVariable = /** @class */ (function () {
        function ExtractPageVariable(variableName) {
            this._variableName = variableName;
            this._handShake = this._generateHandshake();
            this._inject();
            this._data = this._listen();
        }
        Object.defineProperty(ExtractPageVariable.prototype, "data", {
            //@ts-ignore
            get: function () {
                return this._data;
            },
            enumerable: false,
            configurable: true
        });
        // Private
        ExtractPageVariable.prototype._generateHandshake = function () {
            var array = new Uint32Array(5);
            return window.crypto.getRandomValues(array).toString();
        };
        ExtractPageVariable.prototype._inject = function () {
            function propagateVariable(handShake, variableName) {
                var message = { handShake: handShake };
                message[variableName] = window[variableName];
                window.postMessage(message, "*");
            }
            var script = "( " + propagateVariable.toString() + " )('" + this._handShake + "', '" + this._variableName + "');";
            var scriptTag = document.createElement('script');
            var scriptBody = document.createTextNode(script);
            scriptTag.id = 'chromeExtensionDataPropagator';
            scriptTag.appendChild(scriptBody);
            document.body.append(scriptTag);
        };
        ExtractPageVariable.prototype._listen = function () {
            var _this = this;
            return new Promise(function (resolve) {
                window.addEventListener("message", function (_a) {
                    var data = _a.data;
                    // We only accept messages from ourselves
                    if (data.handShake != _this._handShake)
                        return;
                    resolve(data);
                }, false);
            });
        };
        return ExtractPageVariable;
    }());
    // const windowData = new ExtractPageVariable('__PRELOADED_STATE__').data;
    // windowData.then(console.log);
    // windowData.then(data => {
    //    // Do work here
    // });

    var convertToCnNumber = function (d) {
        if (d / 100000000 >= 1) {
            return Math.floor(d / 100000000) + "\u4EBF";
        }
        if (d / 10000 >= 1) {
            return Math.floor(d / 10000) + "\u4E07";
        }
        return Math.floor(d).toString();
    };

    exports.ExtractPageVariable = ExtractPageVariable;
    exports.SimpleEventHandler = SimpleEventHandler;
    exports.convertToCnNumber = convertToCnNumber;
    exports.downFile = downFile;
    exports.extractJSON = extractJSON;
    exports.extractJSONV2 = extractJSONV2;
    exports.getCanvas = getCanvas;
    exports.getUidFromLocalStorage = getUidFromLocalStorage;
    exports.loadScript = loadScript;
    exports.openNewTab = openNewTab;
    exports.promisesReduce = promisesReduce;
    exports.scrollAndExecute = scrollAndExecute;
    exports.timeoutAsync = timeoutAsync;
    exports.timeoutAsyncRej = timeoutAsyncRej;
    exports.uid = uid;
    exports.uniqObjArrByKey = uniqObjArrByKey;
    exports.uniqueArray = uniqueArray;
    exports.uuidv4 = uuidv4;
    exports.waitUntilElementExist = waitUntilElementExist;

    Object.defineProperty(exports, '__esModule', { value: true });

});
