'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var winston$1 = require('winston');
var hpagent = require('hpagent');
var Bottleneck = _interopDefault(require('bottleneck'));

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

function __spreadArrays() {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
}

function uniqueArray(currentData, newData, comparator) {
    return __spreadArrays(currentData, newData.filter(function (i) {
        return !currentData.reduce(function (acc, cur) {
            return acc || comparator(cur, i);
        }, false);
    }));
}
function uniqObjArrByKey(currentData, key) {
    return currentData.reduce(function (acc, cur) {
        return !!acc.find(function (i) { return i[key] === cur[key]; }) ? acc : __spreadArrays(acc, [cur]);
    }, []);
}
var timeoutAsync = function (timeout) { return (new Promise(function (res) { return setTimeout(res, timeout); })); };
var timeoutAsyncRej = function (timeout) { return (new Promise(function (res, rej) { return setTimeout(function () { return rej(new Error("timeoutAsyncRej timeout:" + timeout)); }, timeout); })); };
function uuidv4() {
    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
        var r = Math.random() * 16 | 0, v = c == 'x' ? r : (r & 0x3 | 0x8);
        return v.toString(16);
    });
}
function uid() {
    return (Date.now().toString(36) + Math.random().toString(36).substr(2, 5)).toUpperCase();
    //return '_' + Math.random().toString(36).substr(2, 9);
}
var SimpleEventHandler = /** @class */ (function () {
    function SimpleEventHandler() {
        this.handler = {};
    }
    SimpleEventHandler.prototype.addEventListener = function (evtName, fn) {
        var _this = this;
        var id = uuidv4();
        this.handler[evtName] = __spreadArrays(this.handler[evtName] || [], [{ id: id, fn: fn }]);
        return { unwatch: function () {
                var index = _this.handler[evtName].findIndex(function (i) { return i.id === id; });
                _this.handler[evtName].splice(index, 1);
            }
        };
    };
    SimpleEventHandler.prototype.notify = function (name, value) {
        var evtHandlers = this.handler[name] || [];
        evtHandlers.forEach(function (handler) { return handler.fn(value); });
    };
    return SimpleEventHandler;
}());
function extractJSON(str) {
    var firstOpen = str.indexOf('{');
    var sliced = str.slice(firstOpen);
    // console.log(sliced)
    var i = 0;
    var j = 0;
    for (var _i = 0, sliced_1 = sliced; _i < sliced_1.length; _i++) {
        var s = sliced_1[_i];
        j++;
        if (s === '{') {
            i++;
        }
        else if (s === '}') {
            i--;
        }
        if (i === 0) {
            var os = sliced.slice(0, j);
            return JSON.parse(os);
        }
    }
}
function extractJSONV2(str) {
    var firstOpen = -1, firstClose, candidate;
    firstOpen = str.indexOf('{', firstOpen + 1);
    do {
        firstClose = str.lastIndexOf('}');
        // console.log('firstOpen: ' + firstOpen, 'firstClose: ' + firstClose);
        if (firstClose <= firstOpen) {
            return null;
        }
        do {
            candidate = str.substring(firstOpen, firstClose + 1);
            // console.log('candidate: ' + candidate);
            try {
                var res = JSON.parse(candidate);
                // console.log('...found');
                return [res, firstOpen, firstClose + 1];
            }
            catch (e) {
                // console.log('...failed');
            }
            firstClose = str.substr(0, firstClose).lastIndexOf('}');
        } while (firstClose > firstOpen);
        firstOpen = str.indexOf('{', firstOpen + 1);
    } while (firstOpen != -1);
}

function promisesReduce(promises) {
    var final = [];
    return promises.reduce(function (pre, cur) {
        return pre.then(function () {
            return cur().then(function (result) { return final.push(result); });
        });
    }, Promise.resolve())
        .then(function () { return final; });
}

var fs = require('fs');
var util = require('util');
var accessAsync = util.promisify(fs.access);
var readFile = function (filename, encoding) {
    if (encoding === void 0) { encoding = 'utf-8'; }
    return new Promise(function (resolve, reject) {
        console.info('readFile:', filename);
        fs.readFile(filename, encoding, function (err, data) {
            if (err) {
                console.error(err);
                reject(err);
            }
            else {
                resolve(data);
            }
        });
    });
};
var writeFile = function (path, data, encoding) {
    if (encoding === void 0) { encoding = 'utf-8'; }
    return new Promise(function (resolve, reject) {
        fs.writeFile(path, data, encoding, function (err, result) {
            if (err)
                reject(err);
            else {
                resolve(data);
            }
        });
    });
};
var fileRemove = function (name) {
    return new Promise(function (res, rej) {
        fs.unlink(name, function (err) {
            err ? rej(err) : res();
        });
    });
};
var fileExists = function (name) {
    return accessAsync(name)
        .then(function () { return true; })["catch"](function () { return false; });
};
var listDir = function (path) {
    return new Promise(function (res, rej) {
        fs.readdir(path, function (error, result) {
            console.log(path, result);
            error ? rej(error) : res(result);
        });
    });
};
var rename = function (oldPath, newPath, renameFileIfExist) {
    if (renameFileIfExist === void 0) { renameFileIfExist = false; }
    return new Promise(function (res, rej) {
        fileExists(newPath)
            .then(function (exist) {
            console.log('#rename fileExists:', exist);
            if (!exist) {
                console.log('rename', exist, !exist);
                fs.rename(oldPath, newPath, function (error) {
                    console.log('#rename error:', error);
                    error ? rej(error) : res();
                });
            }
            else {
                if (renameFileIfExist) {
                    console.log('rename if exist');
                    fs.rename(oldPath, newPath + "_" + new Date().getTime(), function (error) {
                        console.log('#rename error:', error);
                        error ? rej(error) : res();
                    });
                }
                else {
                    rej(new Error("#rename dest fileExists:" + newPath));
                }
            }
        });
    });
};
var mkDir = function (path) {
    return new Promise(function (res, rej) {
        fileExists(path)
            .then(function (exist) {
            if (exist) {
                var error = new Error("mkdir " + path + " already exist");
                rej(error);
            }
            else {
                fs.mkdir(path, { recursive: true }, function (error) {
                    if (error) {
                        console.error('mkdir error:', path, error);
                        rej(error);
                    }
                    else {
                        res();
                    }
                });
            }
        })["catch"](function (err) { return rej(err); });
    });
};

/* v1.3
 * add db handler
 * mongo db js driver move to 3
 *
 */
var MongoClient = require("mongodb").MongoClient;
function connectToMongo(url) {
    return __awaiter(this, void 0, void 0, function () {
        var client, e_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    client = new MongoClient(url, {
                        useNewUrlParser: true,
                        useUnifiedTopology: true,
                        poolSize: 20,
                    });
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, 4, 5]);
                    // Connect the client to the server
                    return [4 /*yield*/, client.connect()];
                case 2:
                    // Connect the client to the server
                    _a.sent();
                    // Establish and verify connection
                    // await client.db("admin").command({ ping: 1 });
                    console.log("Connected successfully to server url: " + url);
                    return [2 /*return*/, client]; // if dbName is null return default db
                case 3:
                    e_1 = _a.sent();
                    console.error("Connected to server failed url: " + url, e_1);
                    return [3 /*break*/, 5];
                case 4: return [7 /*endfinally*/];
                case 5: return [2 /*return*/];
            }
        });
    });
}
function add(db, collection, data) {
    return new Promise(function (resolve, reject) {
        var c = db.collection(collection);
        c.insert(data, function (err, result) {
            if (err) {
                reject(err);
            }
            else {
                resolve(result.ops[0]);
            }
        });
    });
}
function find(db, collection, query) {
    return new Promise(function (resolve, reject) {
        var c = db.collection(collection);
        c.find(query).toArray(function (err, result) {
            if (err) {
                reject(err);
            }
            else {
                resolve(result);
            }
        });
    });
}
function findOne(db, collection, query) {
    var c = db.collection(collection);
    return c.findOne(query);
}
function findOneAndUpdate(db, collection, query, data) {
    var c = db.collection(collection);
    return c.updateOne(query, { $set: data }, { upsert: true });
}
function deleteOne(db, collection, params) {
    var c = db.collection(collection);
    return c.deleteOne(params);
}
var Db = /** @class */ (function () {
    function Db(client, dbName) {
        this.db = client.db(dbName);
    }
    Db.prototype.add = function (collection, data) {
        return add(this.db, collection, data);
    };
    Db.prototype.find = function (collection, query) {
        if (query === void 0) { query = {}; }
        return find(this.db, collection, query);
    };
    Db.prototype.findOne = function (collection, query) {
        if (query === void 0) { query = {}; }
        return findOne(this.db, collection, query);
    };
    Db.prototype.update = function (collection, query, update) {
        return findOneAndUpdate(this.db, collection, query, update);
    };
    Db.prototype.deleteOne = function (collection, query) {
        return deleteOne(this.db, collection, query);
    };
    Db.prototype.exist = function (collection, query) {
        return find(this.db, collection, query)
            .then(function (result) {
            if (!!result.length) {
                return Promise.resolve(true);
            }
            else {
                return Promise.resolve(false);
            }
        });
    };
    return Db;
}());
var DbHandler = /** @class */ (function () {
    function DbHandler(url) {
        this.url = url;
        this.client = null;
        this.dbs = {};
        //this.init()
        //setTimeout(this.init.bind(this), 5000)
    }
    DbHandler.prototype.init = function () {
        var _this = this;
        return connectToMongo(this.url)
            .then(function (client) {
            _this.client = client;
            // this.db = db
            return Promise.resolve();
        });
    };
    DbHandler.prototype.getDb = function (dbName) {
        if (!this.dbs[dbName]) {
            this.dbs[dbName] = new Db(this.client, dbName);
        }
        return this.dbs[dbName];
    };
    return DbHandler;
}());

var pThrottle = require('p-throttle');
var tunnel = require('tunnel');
var got = require('got');
var getProxyOptions = function (options) { return ({
    agent: {
        https: tunnel.httpOverHttp({
            proxy: options.proxy
        })
    }
}); };
function getThrotedRequest(numbers, milisecond, startFn) {
    if (startFn === void 0) { startFn = function (_) { }; }
    var requestfunction = function (options) {
        var newOptions = options.proxy && getProxyOptions(options) || {};
        startFn(options, newOptions);
        return new Promise(function (res, rej) {
            timeoutAsync(options.timeout)
                .then(function () { return (rej(new Error("requestTimeout " + options.timeout))); });
            got(options.url, newOptions)
                .then(function (d) { return res(d.body); })["catch"](function (e) { return rej(e); });
        });
    };
    var throttled = pThrottle(requestfunction, numbers, milisecond);
    return throttled;
}

var winston = require('winston');
var fs$1 = require('fs');
var path = require('path');
require('winston-daily-rotate-file');
var combine = winston$1.format.combine, timestamp = winston$1.format.timestamp, label = winston$1.format.label, printf = winston$1.format.printf;
var myFormat = printf(function (info) {
    return info.timestamp + " [" + info.module.padEnd(15, ' ') + "] " + info.level + ":   " + info.message;
});
var logDir = 'logs';
if (!fs$1.existsSync(logDir)) {
    // Create the directory if it does not exist
    fs$1.mkdirSync(logDir);
}
var transport2 = new winston.transports.DailyRotateFile({
    level: 'debug',
    filename: path.join(logDir, '/application-%DATE%.log'),
    datePattern: 'YYYY-MM-DD',
    zippedArchive: true,
    maxSize: '20m',
    maxFiles: '30d'
});
var logger = winston.createLogger({
    level: 'debug',
    //format: winston.format.json(),
    format: combine(timestamp(), myFormat),
    transports: [
        //
        // - Write to all logs with level `info` and below to `combined.log` 
        // - Write all logs error (and below) to `error.log`.
        //
        new winston.transports.File({ filename: path.join(logDir, '/error.log'), level: 'error' }),
        // new winston.transports.File({ filename: path.join(logDir, '/combined.log')})
        transport2
    ]
});
//
// If we're not in production then log to the `console` with the format:
// `${info.level}: ${info.message} JSON.stringify({ ...rest }) `
//
if (process.env.NODE_ENV !== 'production') {
    logger.add(new winston.transports.Console({
        //format: winston.format.simple()
        level: 'info',
        format: winston.format.combine(winston.format.colorize(), 
        //winston.format.simple()
        myFormat),
    }));
}
var addTransport = function (transport) {
    logger.add(transport);
};
var Logger = /** @class */ (function () {
    function Logger(module) {
        this.module = module;
    }
    Logger.prototype.info = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var message = args.map(function (i) { return JSON.stringify(i); }).join(' ');
        logger.info({ module: this.module, message: message });
    };
    Logger.prototype.debug = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var message = args.map(function (i) { return JSON.stringify(i); }).join(' ');
        logger.debug({ module: this.module, message: message });
    };
    Logger.prototype.error = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var message = args.map(function (i) { return JSON.stringify(i); }).join(' ');
        logger.error({ module: this.module, message: message });
    };
    Logger.prototype.warn = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var message = args.map(function (i) { return JSON.stringify(i); }).join(' ');
        logger.warn({ module: this.module, message: message });
    };
    Logger.prototype.log = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        var message = args.map(function (i) { return JSON.stringify(i); }).join(' ');
        logger.warn({ module: this.module, message: message });
    };
    return Logger;
}());
function newLogger(name) {
    return new Logger(name);
}

var crypto = require('crypto');
function createHash(str, algorithm) {
    if (algorithm === void 0) { algorithm = 'md5'; }
    var md5 = crypto.createHash(algorithm); //定义加密方式:md5不可逆,此处的md5可以换成任意hash加密的方法名称；
    md5.update(str);
    var d = md5.digest('hex'); //加密后的值d
    //console.info(`create HASH ${d}`)
    return d;
}

var exec = require('child-process-promise').exec;
var wrappedExec = function (cmd) {
    return exec(cmd)
        .then(function (result) {
        return result.stdout;
    })["catch"](function (err) {
        console.error(err.stderr);
        throw err;
    });
};

var axios = require('axios');
var defaultProxyAgentConfig = {
    keepAlive: true,
    keepAliveMsecs: 1000,
    maxSockets: 256,
    maxFreeSockets: 256,
    timeout: 10000
};
var getHttpAgent = function (getProxy, agentTimeout, logger) { return __awaiter(void 0, void 0, void 0, function () {
    var proxy, ip, port, proxyURI;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, getProxy()];
            case 1:
                proxy = _a.sent();
                ip = proxy.ip, port = proxy.port;
                logger && logger.info("#getHttpAgent with proxy, ip: " + ip + " port: " + port);
                proxyURI = ("http://" + ip + ":" + port);
                return [2 /*return*/, new hpagent.HttpsProxyAgent(__assign(__assign({}, defaultProxyAgentConfig), { timeout: agentTimeout, proxy: proxyURI }))];
        }
    });
}); };
var req = function (_a) {
    var logger = _a.logger, url = _a.url, headers = _a.headers, _b = _a.retry, retry = _b === void 0 ? 3 : _b, getProxy = _a.getProxy, _c = _a.timeout, timeout = _c === void 0 ? 10000 : _c, _d = _a.agentTimeout, agentTimeout = _d === void 0 ? 10000 : _d, proxyAgentConfig = _a.proxyAgentConfig, checkData = _a.checkData // if exist and failes, will retry
    ;
    return __awaiter(void 0, void 0, void 0, function () {
        var errors, _i, _e, i, instance, _f, _g, _h, _j, data, message, e_1, message;
        return __generator(this, function (_k) {
            switch (_k.label) {
                case 0:
                    logger && logger.info("#reqNew: url:" + url + " header:" + JSON.stringify(headers) + " retry:" + retry + " agentTimeout:" + agentTimeout + " timeout:" + timeout);
                    errors = [];
                    _i = 0, _e = Array(retry).fill(0).map(function (i, index) { return index; });
                    _k.label = 1;
                case 1:
                    if (!(_i < _e.length)) return [3 /*break*/, 9];
                    i = _e[_i];
                    logger && logger.info("reqNew try index: #" + (i + 1));
                    _g = (_f = axios).create;
                    _h = {
                        baseURL: url,
                        timeout: timeout
                    };
                    if (!getProxy) return [3 /*break*/, 3];
                    return [4 /*yield*/, getHttpAgent(getProxy, agentTimeout, logger)];
                case 2:
                    _j = _k.sent();
                    return [3 /*break*/, 4];
                case 3:
                    _j = null;
                    _k.label = 4;
                case 4:
                    instance = _g.apply(_f, [(_h.httpsAgent = _j,
                            _h.validateStatus = function () { return true; } // Accepts all status codes
                        ,
                            _h)]);
                    _k.label = 5;
                case 5:
                    _k.trys.push([5, 7, , 8]);
                    return [4 /*yield*/, instance.get()];
                case 6:
                    data = _k.sent();
                    if (checkData) {
                        if (checkData(data.data)) {
                            logger && logger.info("reqNew checkData success, return ");
                            return [2 /*return*/, data];
                        }
                        else {
                            message = "reqNew checkData failed: data: " + JSON.stringify(data.data) + ", continue";
                            logger && logger.info(message);
                            errors.push(message);
                            return [3 /*break*/, 8];
                        }
                    }
                    return [2 /*return*/, data];
                case 7:
                    e_1 = _k.sent();
                    message = "reqNew failed: " + e_1.message;
                    logger && logger.info(message);
                    errors.push(message);
                    return [3 /*break*/, 8];
                case 8:
                    _i++;
                    return [3 /*break*/, 1];
                case 9:
                    logger && logger.info("all request fails, " + JSON.stringify(errors));
                    throw new Error("all request fails");
            }
        });
    });
};

var DEFAULT_CONFIG = {
    maxQueueSize: null,
    rateLimit: null //  if set 'maxConnections' will forced to 1
};
var Crawler = /** @class */ (function () {
    function Crawler(config) {
        var _this = this;
        this.queue = function (config) { return __awaiter(_this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                return [2 /*return*/, this.limiter.schedule(req, config)];
            });
        }); };
        this.onDone = function (fn) {
            _this.onDoneFn = fn;
        };
        this.jobs = function () { return __awaiter(_this, void 0, void 0, function () {
            var done, running;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.limiter.done()];
                    case 1:
                        done = _a.sent();
                        return [4 /*yield*/, this.limiter.running()];
                    case 2:
                        running = _a.sent();
                        return [2 /*return*/, [running, done]];
                }
            });
        }); };
        this.config = config;
        var newConfig = __assign(__assign({}, DEFAULT_CONFIG), config);
        this.limiter = new Bottleneck({
            maxConcurrent: newConfig.maxQueueSize,
            minTime: newConfig.rateLimit
        });
        this.limiter.on("idle", function () {
            // This will be called when `limiter.empty()` is `true` and `limiter.running()` is `0`.
            _this.onDoneFn && _this.onDoneFn();
        });
    }
    return Crawler;
}());

var convertToCnNumber = function (d) {
    if (d / 100000000 >= 1) {
        return Math.floor(d / 100000000) + "\u4EBF";
    }
    if (d / 10000 >= 1) {
        return Math.floor(d / 10000) + "\u4E07";
    }
    return Math.floor(d).toString();
};

exports.Crawler = Crawler;
exports.Db = Db;
exports.DbHandler = DbHandler;
exports.SimpleEventHandler = SimpleEventHandler;
exports.addTransport = addTransport;
exports.convertToCnNumber = convertToCnNumber;
exports.createHash = createHash;
exports.extractJSON = extractJSON;
exports.extractJSONV2 = extractJSONV2;
exports.fileExists = fileExists;
exports.fileRemove = fileRemove;
exports.getThrotedRequest = getThrotedRequest;
exports.listDir = listDir;
exports.logger = logger;
exports.mkDir = mkDir;
exports.newLogger = newLogger;
exports.promisesReduce = promisesReduce;
exports.readFile = readFile;
exports.rename = rename;
exports.req = req;
exports.timeoutAsync = timeoutAsync;
exports.timeoutAsyncRej = timeoutAsyncRej;
exports.uid = uid;
exports.uniqObjArrByKey = uniqObjArrByKey;
exports.uniqueArray = uniqueArray;
exports.uuidv4 = uuidv4;
exports.wrappedExec = wrappedExec;
exports.writeFile = writeFile;
